import gridIconStr from '../style/icon/grid.svg';
import { LabIcon } from '@jupyterlab/ui-components';

export const gridIcon = new LabIcon({
  name: 'jupytercad:grid-icon',
  svgstr: gridIconStr
});
